define([
    "dojo/_base/array",
    "dojo/_base/declare",
    "dojo/dom-construct",
    "dojo/when",
    "dijit/layout/TabContainer",
    "dijit/registry",
    "epi/dependency",
    "epi-cms/core/ContentReference",
    "epi-cms/_ContentContextMixin",
    "epi/shell/tooltipHider",
    "dojo/text!./templates/EditLayoutContainer.html"
], function (
    array,
    declare,
    domConstruct,
    when,
    TabContainer,
    registry,
    dependency,
    ContentReference,
    _ContentContextMixin,
    tooltipHider,
    template
) {

    return declare([TabContainer, _ContentContextMixin], {
        // summary:
        //     Widget that contains a group form items widgets, aimed to apply the html formatting specific to a group
        //
        // tags:
        //      internal

        templateString: template,

        postMixInProperties: function () {
            this.inherited(arguments);

            this.profile = this.profile || dependency.resolve("epi.shell.Profile");
            // This container does not allow doing layout children and only support top tabs because we use relative position when laying out.
            // The reason behind is that absolute position make the container zero height when it is laid out without size constraints.
            this.doLayout = false;
            this.tabPosition = "top";
        },

        startup: function () {
            this.inherited(arguments);
            when(this.getCurrentContext(), this._trySelectStoredTab.bind(this));
            tooltipHider(this);

            // The top pane container does not automatically start its' widgets, so we do it manually
            var topPaneContents = registry.findWidgets(this.topPaneContainerNode);
            array.forEach(topPaneContents, function (widget) {
                !widget.started && widget.startup();
            });
        },

        addChild: function (widget, index) {
            // summary:
            //     Add a child o the container.
            // widget: dijit/_Widget
            //     The widget to add
            // index: Number
            //     The position
            // tags:
            //     public

            if (this._destroyed) {
                return;
            }

            if (widget.region === "top") {
                this.own(widget.placeAt(this.topPaneContainerNode));
            } else {
                this.inherited(arguments);
            }
        },

        removeChild: function (widget) {
            // Overrides _Container.removeChild() to do layout

            if (widget.region === "top") {
                // Return if we are being destroyed
                if (this._descendantsBeingDestroyed) {
                    return;
                }

                // Otherwise, do layout again
                if (this._started) {
                    this.layout();
                }
            } else {
                this.inherited(arguments);
            }
        },

        layout: function () {
            // Overrides TabContainer.layout().
            // Configure the content pane to take up all the space except for where the tabs and the top panel are.

            if (!this._contentBox || typeof (this._contentBox.l) == "undefined") {
                return;
            }

            // layout the top panel first
            var topPaneContents = registry.findWidgets(this.topPaneContainerNode);
            array.forEach(topPaneContents, function (widget) {
                widget.resize();
            });

            return this.inherited(arguments);
        },

        selectChild: function (widget) {
            // summary:
            //     Select child widget or group.
            // widget: dijit/_Widget
            //     The child widget or group
            // tags:
            //     public

            if (!widget) {
                return;
            }

            if (widget._type === "group") {
                this._saveSelectedTab(widget);
            }

            if (widget.region === "top") {
                return;
            } else {
                // After inherited finishes we call layout to reflow the content
                // in case a scrollbar has appeared on tab change, and then we pass on the result
                return when(this.inherited(arguments)).then(function (result) {
                    this.layout();
                    return result;
                }.bind(this));
            }
        },

        _saveSelectedTab: function (widget) {
            when(this.getCurrentContext(), function (context) {
                this.profile.set(this._getSelectedTabProfileKey(context.id), this.getChildren().indexOf(widget));
            }.bind(this));
        },

        _getSelectedTabProfileKey: function (contentLink) {
            var contentId = new ContentReference(contentLink).createVersionUnspecificReference().toString();
            return "EditFormContainer_selectedTab_" + contentId;
        },

        _trySelectStoredTab: function (context) {
            when(this.profile.get(this._getSelectedTabProfileKey(context.id))).then(function (selectedTabIndex) {
                if (!selectedTabIndex) {
                    return;
                }

                var children = this.getChildren();

                if (selectedTabIndex < children.length) {
                    this.selectChild(children[selectedTabIndex]);
                }
            }.bind(this));
        }
    });
});
