define([
    // dojo
    "dojo/_base/declare",
    "dojo/aspect",

    // dijit
    "dijit/_TemplatedMixin",
    "dijit/_WidgetBase",
    "dijit/_WidgetsInTemplateMixin",

    // epi.shell
    "epi/shell/widget/_ValueRequiredMixin",
    "epi/shell/widget/_FocusableMixin",

    // epi.cms
    "epi-cms/widget/_HasChildDialogMixin",
    "epi-cms/widget/_Droppable"
], function (
    // dojo
    declare,
    aspect,

    // dijit
    _TemplatedMixin,
    _WidgetBase,
    _WidgetsInTemplateMixin,

    // epi.shell
    _ValueRequiredMixin,
    _FocusableMixin,

    // epi.cms
    _HasChildDialogMixin,
    _Droppable
) {

    return declare([_WidgetBase, _TemplatedMixin, _WidgetsInTemplateMixin, _ValueRequiredMixin, _FocusableMixin, _HasChildDialogMixin, _Droppable], {
        // summary:
        //    Represents the selector wrapper.
        // tags:
        //      internal xproduct

        templateString: "<div><div data-dojo-attach-point='container,dropAreaNode'></div></div>",

        selectorClass: null,

        value: null,

        postMixInProperties: function () {
            // summary:
            //    Initialize properties
            // tags:
            //    protected

            this.inherited(arguments);
            // Ignore value here then set later via set("value") method.
            this.selector = new this.selectorClass(Object.assign({}, this.params, { value: null }, this._getSelectorParams()));
            this.own(this.selector);
        },

        buildRendering: function () {
            // summary:
            //    Creates and append the selector to the container.
            // tags:
            //    protected

            this.inherited(arguments);
            this.selector.placeAt(this.container);
        },

        postCreate: function () {
            this.inherited(arguments);

            this.own(aspect.after(this.selector, "onChange", function (value) {
                this._onSelectorValueChange(value);
                this.onChange(this.get("value"));
            }.bind(this), true));
        },

        onChange: function (value) {
            // summary:
            //      An extension point invoked when the value has changed.
            // tags:
            //      public
        },

        _onSelectorValueChange: function (val) {
            // summary:
            //      Callback whenever selector's value is changed.
            //
            // tags:
            //      protected abstract

            // should implement by usage or inheritance class.
        },

        _getSelectorParams: function () {
            // summary:
            //      Retrieves the selector's params.
            //
            // tags:
            //      protected abstract

            // should implement by usage or inheritance class.
            return {};
        },

        focus: function () {
            this.selector && this.selector.focus();
        },

        _set: function (name, value) {
            this.inherited(arguments);
            if (name === "value") {
                this._setValue(value);
                return;
            }
            this.selector && this.selector.set(name, value);
        },

        _setValueAttr: function (value) {
            return this._setValue(value);
        },

        _setValue: function (value) {
            this.value = value;
            this.selector && this.selector.set("value", value);
        },

        _get: function (name, value) {
            if (name === "value") {
                return this._getValue(value);
            }
            return this.selector && this.selector.get(name, value);
        },

        _getValueAttr: function () {
            return this._getValue();
        },

        _getValue: function () {
            return this.value;
        },

        _getPermanentLinkAttr: function () {
            return this.selector && this.selector.get("permanentLink");
        },

        onDrop: function () {
            this._triggerSave();
        },

        _triggerSave: function () {
            this.onChange && this.onChange(this.value);
            this._hasBeenBlurred = true;
            this.validate(true);
            this.onBlur();
        }
    });
});
