define([
    // dojo
    "dojo/_base/declare",
    "dojo/aspect",
    "dojo/on",
    "dojo/Deferred",
    "dojo/Stateful",
    "dojo/string",
    "dijit/Destroyable",

    // epi.shell
    "epi/epi",
    "epi/shell/widget/ContextMenu",
    "epi/shell/command/_CommandProviderMixin",
    "epi/shell/command/withConfirmation",
    "epi/shell/TypeDescriptorManager",
    "epi/shell/DialogService",
    "epi/shell/widget/dialog/Confirmation",

    // epi.cms
    "epi-cms/dgrid/formatters",
    "epi-cms/ApplicationSettings",

    // commands
    "epi-cms/contentediting/command/BlockEdit",
    "epi-cms/contentediting/command/BlockInlineEdit",
    "epi-cms/contentediting/command/BlockRemove",
    "epi-cms/widget/command/DefaultContentSelectorPlugAndPlayCommand",

    // resources
    "epi/i18n!epi/cms/nls/episerver.cms.widget.contentselector",

    // template
    "dojo/text!./templates/_ContentSelectorActionsMixin.html"
], function (
    // dojo
    declare,
    aspect,
    on,
    Deferred,
    Stateful,
    dojoString,
    Destroyable,

    // epi.shell
    epi,
    ContextMenu,
    _CommandProviderMixin,
    withConfirmation,
    TypeDescriptorManager,
    DialogService,
    Confirmation,

    // epi.cms
    formatters,
    ApplicationSettings,

    // commands
    EditCommand,
    BlockInlineEdit,
    RemoveCommand,
    DefaultContentSelectorPlugAndPlayCommand,

    // resources
    localization,

    // template
    template
) {

    return declare([_CommandProviderMixin], {
        // summary:
        //    Represents the widget to select ContentReference with actions.
        // tags:
        //      internal product

        baseClass: "epi-content-selector-with-actions",

        templateString: template,

        resources: localization,

        modelClass: declare([Stateful, Destroyable]),
        model: null,

        iconClass: null,

        postMixInProperties: function () {
            // summary:
            //		Initialize properties
            // tags:
            //    protected

            this.inherited(arguments);

            // setup model and context menu
            var that = this;
            this.own(this.model = this.model || new this.modelClass({
                readOnly: that.readOnly,
                remove: function () {
                    that.clearValue({ preventDefault: function () { } });
                }
            }));
        },

        buildRendering: function () {
            // summary:
            //		Update text and setup context menu.
            // tags:
            //    protected

            this.inherited(arguments);
            this.own(
                on(this.displayNode, "click", function (evt) {
                    evt.preventDefault();
                    evt.stopPropagation();

                    this.focus();
                }.bind(this))
            );

            this._setupContextMenu();
        },

        _getChangeSelectionCommand: function () {
            var dialogSettings = Object.assign({
                title: this.title,
                dialogClass: "epi-dialog-portrait epi-content-selector-plugnplay-dialog",
                destroyOnHide: true
            }, this.dialogSettings || {});

            var contentSelectorDialogSettings = Object.assign({
                roots: [ApplicationSettings.rootPage],
                allowedTypes: ["episerver.core.icontent", "episerver.core.icontentdata"],
                canSelectOwnerContent: false,
                getCurrentValue: function () {
                    return undefined;
                },
                isInQuickEditMode: this.isInQuickEditMode,
                quickEditBlockId: this.quickEditBlockId
            }, this.contentSelectorDialogSettings || {});

            var changeSelection = new DefaultContentSelectorPlugAndPlayCommand({
                label: epi.resources.action.replace,
                canExecute: !this.readOnly,
                dialogSettings: dialogSettings,
                contentSelectorDialogSettings: contentSelectorDialogSettings
            });

            this.own(
                aspect.after(changeSelection, "onContentSelectorChange", this._onContentSelectorChange.bind(this), true),
                aspect.after(changeSelection, "onSearchResultSelected", this._onSearchResultSelected.bind(this), true),
                aspect.after(changeSelection, "onDialogExecute", this._onDialogExecute.bind(this), true),
                aspect.after(changeSelection, "onDialogHide", this._onDialogHide.bind(this), true),
                aspect.after(changeSelection, "onDialogShow", this._onDialogShow.bind(this), true)
            );

            return changeSelection;
        },

        _setupContextMenu: function () {
            // summary: set up the context menu
            //
            // tags:
            //      public

            var that = this;
            var editCommand = new EditCommand({ category: null });

            this.commands = [
                withConfirmation(editCommand, function (settings) {
                    var dialogStack = DialogService.getEnclosingDialog(that), dialog = dialogStack && dialogStack.dialog;
                    if (!dialog) {
                        return;
                    }

                    if (!DialogService.hasPendingChanges(dialogStack, epi.compareOptions.skipEmptyProperties)) {
                        dialog.onCancel();
                        return;
                    }

                    var deferred = new Deferred();

                    var confirmationDialog = new Confirmation({
                        title: that.resources.command.blockedit.title,
                        description: dojoString.substitute(that.resources.command.blockedit.description, [dialog.titleNode.textContent.trim()]),
                        confirmActionText: that.resources.command.blockedit.action.yes,
                        cancelActionText: that.resources.command.blockedit.action.no
                    });

                    that.own(
                        confirmationDialog,
                        // TODO: Temporary solution until we apply new dialog layout (CMS-20406).
                        aspect.around(confirmationDialog, "getActions", function (originalMethod) {
                            return function () {
                                var actions = originalMethod.apply(this, arguments);
                                actions[0].settings = { "class": "epi-plain-button" }; // yes
                                actions[1].settings = { "class": "Salt" }; // no
                                return actions;
                            }.bind(confirmationDialog);
                        }.bind(confirmationDialog)),

                        aspect.after(confirmationDialog, "onAction", function (confirm) {
                            if (confirm) {
                                // force dialog stacks close
                                DialogService.hideStack();
                                deferred.resolve();
                            } else {
                                dialog.resize(); // restore parent dialog underlay: BlockInlineEdit has custom underlay position.
                                deferred.cancel();
                            }
                        }.bind(confirmationDialog), true)
                    );

                    confirmationDialog.startup();
                    confirmationDialog.show();
                    confirmationDialog.resize(); // force underlay cover all owner document.

                    return deferred.promise;
                }.bind(editCommand)),
                new BlockInlineEdit(),
                this._getChangeSelectionCommand(),
                new RemoveCommand()
            ];

            this.commands.forEach(function (command) {
                this.own(command);
            }, this);

            this.own(
                this.commands,
                this.contextMenu = new ContextMenu({
                    onClose: function () {
                        that.displayNode.classList.remove("dijitFocused", "epi-content-selector-with-actionsActive");
                    }
                }),
                on(this.iconNodeMenu, "click", function (evt) {
                    evt.preventDefault();
                    evt.stopPropagation();

                    this.displayNode.classList.add("dijitFocused", "epi-content-selector-with-actionsActive");
                    this.contextMenu && this.contextMenu.scheduleOpen(evt.target, null, { x: evt.pageX, y: evt.pageY });
                }.bind(this))
            );

            this.contextMenu.addProvider(this);
        },

        _updateDisplayNode: function (content) {
            // summary:
            //      Overridden to initialize ui when value is set.

            this.inherited(arguments);

            // Reset text + icon to empty state.
            this.iconNode.className = this.iconNode.className.replace(this.iconClass, "");
            this.resourceName.title = this.resources.helptext;

            if (content) {
                this.iconClass = formatters.contentIconClass(content.typeIdentifier);
                this.iconNode.className += (" " + this.iconClass);

                this.resourceName.title = formatters.title(this.resourceName.textContent.trim(), content.contentLink, TypeDescriptorManager.getResourceValue(content.typeIdentifier, "name"));

                for (var propertyName in content) {
                    this.model.set(propertyName, content[propertyName]);
                }
            }

            this.updateCommandModel(content ? this.model : null);
        }
    });
});
