define([
    "dojo/_base/declare",
    "dojo/on",
    "dojo/when",

    "dijit/_TemplatedMixin",
    "dijit/_WidgetsInTemplateMixin",
    "dijit/layout/_LayoutWidget",

    // Shell
    "epi",
    "epi/dependency",
    "epi/shell/command/_Command",
    "epi/shell/widget/dialog/Dialog",

    // CMS
    "epi-cms/contentediting/DialogPositionAdjust",
    "epi-cms/_ContentContextMixin",
    "epi-cms/widget/ContentTypeList",
    "epi-cms/ApplicationSettings",
    "epi-cms/contentediting/inline-editing/CreateNewInlineBlockEditFormContainer",
    "epi-cms/contentediting/inline-editing/InlineEditBlockDialog",

    "dojo/text!./../templates/CreateInlineBlock.html",

    "epi/i18n!epi/cms/nls/episerver.cms.contentediting.editors.contentarea",
    "epi/i18n!epi/cms/nls/episerver.shared.action",
    "epi/i18n!epi/cms/nls/episerver.cms.components.createblock",
    "epi/i18n!epi/cms/nls/contenttypes.blockdata"
],

function (
    declare,
    on,
    when,

    _TemplatedMixin,
    _WidgetsInTemplateMixin,
    _LayoutWidget,

    // Shell
    epi,
    dependency,
    _Command,
    Dialog,

    // CMS
    DialogPositionAdjust,
    _ContentContextMixin,
    ContentTypeList,
    applicationSettings,
    CreateNewInlineBlockEditFormContainer,
    InlineEditBlockDialog,

    template,

    contentAreaRes,
    sharedResources,
    createBlockRes,
    blockDataRes
) {
    var formIdentifier = "episerver.forms.implementation.elements.formcontainerblock";

    var SimpleContentTypeList = declare([ContentTypeList], {
        _getSuggestedContentTypes: function () {
            return [];
        },
        //TODO: Fix support in EPiServer Forms package, for now we disable the command for forms
        _getAvailableContentTypes: function () {
            var result = this.inherited(arguments);
            return when(result).then(function (types) {
                return types.filter(function (type) {
                    return type.typeIdentifier !== formIdentifier;
                });
            });
        }
    });

    var CreateInlineBlock = declare([_LayoutWidget, _TemplatedMixin, _WidgetsInTemplateMixin], {
        templateString: template,

        createBlockRes: createBlockRes,

        getName: function () {
            return this.nameTextBox.get("value");
        },

        setName: function (defaultName) {
            this.nameTextBox.set("value", defaultName);
            this.nameTextBox.focus();
            this.nameTextBox.textbox.select();
        }
    });

    return declare([_Command, _ContentContextMixin], {
        // tags:
        //      internal

        // canExecute: [Boolean]
        //      Default value indicate that command can execute
        canExecute: true,

        // value: [String]
        //      Default select content
        value: null,

        // allowedTypes: [public] Array
        //      The types which are allowed for the given property. i.e used for filtering based on AllowedTypesAttribute
        allowedTypes: null,

        // restrictedTypes: [public] Array
        //      The types which are restricted.
        restrictedTypes: null,

        // suppressOverlayAdjustments: [public] Boolean
        //      Flag which indicates that all overlay adjustments should not be performed. It might be useful when showing another overlay or dialog (e.g. TinyMCE full screen)
        suppressOverlayAdjustments: false,

        postscript: function () {
            this.inherited(arguments);
            this._dialogPositionAdjust = new DialogPositionAdjust();
            this.own(this._dialogPositionAdjust);

            this.contextService = this.contextService || dependency.resolve("epi.shell.ContextService");
        },

        contextChanged: function (ctx, callerData) {
            // summary:
            //      On context change
            // tags:
            //      protected

            this.set("isAvailable", this._isContentContext(ctx));
            this.inherited(arguments);
        },

        _execute: function () {
            // summary:
            //		Executes this command; ...
            // tags:
            //		protected

            when(this.getCurrentContent(), function (currentContent) {
                if (!currentContent) {
                    return;
                }

                // when creating nest block in quick edit
                var contentLink = currentContent.contentLink || currentContent.id;

                // Get the context store for more information about the model
                // We need to append the ?uri= because the request chain can not handle URI's in URI's
                when(this.contextService.query({uri: "epi.cms.contentdata:///" + contentLink}), function (context) {
                    if (!context) {
                        return;
                    }
                    this._createLocalBlock(currentContent, contentLink);
                }.bind(this));
            }.bind(this));
        },

        _createInlineForm: function (content, selectedType) {
            // summary:
            //      Create an inline form for a specific block type
            //
            // content:
            //      The content which will be the parent of the new block
            // selectedType:
            //      Selected block type
            //
            // tags:
            //      private

            var form = new CreateNewInlineBlockEditFormContainer({
                autoPublish: true,
                addToDestination: this.model
            });

            return form.reloadMetadata(content, selectedType.id).then(function () {
                return form;
            });
        },

        _positionDialogOnTopOfMainContent: function (dialog) {
            // summary:
            //      Content Types List will cover the main editing area with some small padding
            //
            // dialog:
            //      Dialog which will cover the editing area
            //
            // tags:
            //      private

            var mainContent = document.querySelector(".epi-main-content");
            if (mainContent) {
                var dimensions = mainContent.getBoundingClientRect();
                var padding = 40;
                dialog.domNode.style.left = dimensions.left + padding + "px";
                dialog.domNode.style.top = dimensions.top + padding + "px";
                dialog.domNode.style.width = dimensions.width - 2 * padding + "px";
                dialog.domNode.style.height = dimensions.height - 2 * padding + "px";
                dialog.domNode.style.maxWidth = 880 + "px";
                dialog.containerNode.style.maxHeight = "calc(100% - 100px)";
            }
        },

        _createLocalBlock: function (content, contentLink) {
            var _this = this;
            var dialog;

            var restrictedTypes = this.restrictedTypes;
            var createInlineBlock;
            var contentTypeList = new SimpleContentTypeList({
                allowedTypes: this.allowedTypes,
                restrictedTypes: restrictedTypes,
                localAsset: true,
                parentLink: contentLink,
                onContentTypeSelected: function (selectedType) {
                    when(_this._createInlineForm(content, selectedType)).then(function (form) {
                        form.placeAt(createInlineBlock.propertiesForm);
                        contentTypeList.destroy();

                        function updateMainCommandVisibility() {
                            if (!dialog) {
                                return;
                            }

                            var formValue = form.get("value");
                            var allValuesAreFalsy = !formValue || Object.values(formValue).every(function (x) {
                                return epi.isEmpty(x);
                            });

                            dialog.toggleDisabledSaveButton(!form.get("isDirty") || allValuesAreFalsy);
                        }
                        _this.own(on(dialog, "execute", function () {
                            form.saveForm(selectedType, createInlineBlock.getName());
                            dialog.hide();
                        }));
                        _this.own(on(dialog, "hide", function () {
                            _this._dialogPositionAdjust.cleanup();
                        }));
                        dialog.own(form);

                        _this._dialogPositionAdjust.adjustDialogPosition(dialog);

                        dialog.toggleDisabledSaveButton(false);
                        updateMainCommandVisibility();

                        dialog.own(on(form, "isDirty", updateMainCommandVisibility.bind(form)));

                        if (!Object.prototype.hasOwnProperty.call(applicationSettings.inlineBlockNameProperties, selectedType.name.toLowerCase())) {
                            var title = sharedResources["new"] + " " + selectedType.localizedName;
                            createInlineBlock.namePanel.classList.remove("dijitHidden");
                            createInlineBlock.setName(title);
                        }
                    });
                }
            });

            _this.own(contentTypeList.watch("shouldSkipContentTypeSelection", function () {
                dialog = new InlineEditBlockDialog({
                    // eslint-disable-next-line no-undef
                    acknowledgeActionText: epi.resources.action.cancel,
                    confirmActionText: epi.resources.action.create,
                    dialogClass: "epi-dialog-portrait inline-edit-dialog inline-create-dialog",
                    title: sharedResources["new"] + " " + contentAreaRes.inline + " " + blockDataRes.name,
                    suppressOverlayAdjustments: _this.suppressOverlayAdjustments,
                    saveLabel: sharedResources.create
                });

                var contextService = _this.contextService;
                dialog.on("cancel", function () {
                    if (!contextService.currentContext) {
                        contextService.currentContext = {};
                    }
                    contextService.currentContext.currentMode = undefined;
                });

                createInlineBlock = new CreateInlineBlock({}, dialog.content, "last");
                contentTypeList.placeAt(createInlineBlock.contentTypeList);

                dialog.show();
                dialog.toggleDisabledSaveButton(true);
                _this._positionDialogOnTopOfMainContent(dialog);
                _this.own(dialog);
                contentTypeList._suggestedContentTypes.setVisibility(false);
                createInlineBlock.own(contentTypeList);
            }));
            contentTypeList.set("requestedType", "episerver.core.blockdata");
            contentTypeList.refresh();
        }
    });
});
