define([
    // dojo
    "dojo/_base/declare",
    "dojo/on",
    "dojo/aspect",

    // diji
    "dijit/Destroyable",

    // epi.shell
    "epi/shell/command/_Command",
    "epi/shell/widget/dialog/ContextAwareDialog",

    // epi-cms
    "epi-cms/widget/ContentSelectorDialog",
    "epi-cms/component/_ShowAllLanguagesMixin",
    "epi-cms/ApplicationSettings"
], function (
    // dojo
    declare,
    on,
    aspect,

    // dijit
    Destroyable,

    // epi.shell
    _Command,
    ContextAwareDialog,

    // epi-cms
    ContentSelectorDialog,
    _ShowAllLanguagesMixin,
    ApplicationSettings
) {
    return declare([_Command, _ShowAllLanguagesMixin, Destroyable], {
        // summary:
        //    Default command class used in ContentSelectorPlugAndPlay widget.
        // tags:
        //      private xproduct

        contentSelectorDialogSettings: null,
        dialogSettings: null,

        dialog: null,
        contentSelectorDialog: null,

        onContentSelectorChange: function (/*object*/value) {
            // summary:
            //    Handle content selector inside dialog is changed value.
            // tags:
            //    public callback
        },

        onSearchResultSelected: function (/*object*/value) {
            // summary:
            //    Handle a search result is selected.
            // tags:
            //    public callback
        },

        onDialogExecute: function (/*object*/value) {
            // summary:
            //    Handle dialog execute
            // tags:
            //    public callback
        },

        onDialogShow: function () {
            // summary:
            //    Handle dialog show
            // tags:
            //    public callback
        },

        onDialogHide: function () {
            // summary:
            //    Handle dialog hide
            // tags:
            //    public callback
        },

        _execute: function () {
            this.componentId = ApplicationSettings.mainNavigationComponentId;
            this.loadShowAllLanguages().then(function (showAllLanguages) {
                this._buildDialog(showAllLanguages);
                var currentValue = this.contentSelectorDialogSettings.getCurrentValue();
                currentValue && this.contentSelectorDialog.set("value", currentValue);
                !currentValue && !this.contentSelectorDialog.setInitialValue && this.contentSelectorDialog.set("value", undefined);
                this.dialog.definitionConsumer.setItemProperty(this.dialog._okButtonName, "disabled", !currentValue);
                this.dialog.show();
                !currentValue && this.contentSelectorDialog.setInitialValue && this.contentSelectorDialog.setInitialValue();
            }.bind(this));
        },

        _buildDialog: function (showAllLanguages) {
            // summary:
            //    Create content tree dialog
            // tags:
            //    protected

            // Verifies that the dialog instance and its dom node existing or not
            if (this.dialog && this.dialog.domNode) {
                return;
            }

            var DialogClass = this.dialogSettings.dialogContentClass || ContentSelectorDialog;
            this.contentSelectorDialog = new DialogClass(Object.assign({}, this.contentSelectorDialogSettings, { showAllLanguages: showAllLanguages }));

            this.dialog = new ContextAwareDialog(Object.assign({ content: this.contentSelectorDialog }, this.dialogSettings));

            this.dialog.own(this.contentSelectorDialog);
            var callbackWithValue = function (callbackName, closeDialog) {
                var value = this.contentSelectorDialog.get("value");
                callbackName && this[callbackName].call(this, value);
                this.dialog && this.dialog.definitionConsumer && this.dialog.definitionConsumer.setItemProperty(this.dialog._okButtonName, "disabled", !value);
                closeDialog && this.dialog && this.dialog.hide();
            }.bind(this);
            this.own(
                this.dialog,
                on(this.contentSelectorDialog, "data-changed", function (value) {
                    this.dialog.definitionConsumer.setItemProperty(this.dialog._okButtonName, "disabled", !value);
                }.bind(this)),
                aspect.after(this.contentSelectorDialog, "onChange", callbackWithValue.bind(this, "onContentSelectorChange", false), true),
                aspect.after(this.contentSelectorDialog, "onSearchResultSelected", callbackWithValue.bind(this, "onSearchResultSelected", true), true),
                aspect.after(this.dialog, "onExecute", callbackWithValue.bind(this, "onDialogExecute"), true),
                aspect.after(this.dialog, "onHide", this.onDialogHide.bind(this), true),
                aspect.after(this.dialog, "onShow", this.onDialogShow.bind(this), true)
            );

            this.dialog.startup();
        }
    });
});
